/*--------------------------------------------------------------------------------------+
|
|     $Source: MstnExamples/View/DisplayRule/DisplayRuleutils.cpp $
|
|  $Copyright: (c) 2018 Bentley Systems, Incorporated. All rights reserved. $
|
+--------------------------------------------------------------------------------------*/
#include "displayruleincludes.h"
#include <Mstn/ISessionMgr.h>
#include <DgnPlatform/DisplayRuleActions.h>
#include <DgnPlatform/CustomItemType.h>
#include <ECObjects/ECSchema.h>
#include <iterator>
#include "displayruleutils.h"

USING_NAMESPACE_BENTLEY_DGNPLATFORM;
USING_NAMESPACE_BENTLEY_MSTNPLATFORM;
USING_NAMESPACE_BENTLEY_MSTNPLATFORM_ELEMENT;

#define   ELEMENTDISPLAY        L"ElementDisplay"
#define   FILLCOLOROVERRIDE     L"FillColorOverride"
#define   COLOROVERRIDE         L"ColorOverride"
#define   ELEMENTPRIORITY       L"ElementPriority"
#define   TRANSPERRENCY         L"TransparencyOverride"
#define   STYLE                 L"StyleOverride"

/*---------------------------------------------------------------------------------**//**
* @description   Shows formatted output message in Message Center
* @bsimethod                                                              Bentley Systems
+---------------+---------------+---------------+---------------+---------------+------*/
WString  DisplayRuleUtils::GetInternalName (WStringCR	name)
{
  WString  internalName;
  ECN::ECNameValidation::EncodeToValidName (internalName, name);
  return   internalName;
}
void DisplayRuleUtils::OutputMessageInMessageCenter(WStringCR message)
{
	OutputMessageInMessageCenter (message.c_str ());
}
void DisplayRuleUtils::OutputMessageInMessageCenter(WCharCP msg)
{
	mdlOutput_messageCenter(OutputMessagePriority::Info, msg, msg, OutputMessageAlert::None);
}
void DisplayRuleUtils::OutputMessageInMessageCenter(UInt32 groupMsgId, UInt32 msgId)
{
	WString message;
	mdlResource_loadWString(message, NULL, groupMsgId, msgId);
	OutputMessageInMessageCenter (message.c_str ());
}

/*---------------------------------------------------------------------------------**//**
* @description   Creates Display Rule Set and adds in active dgn file
* When creating a rule for an Item Type, we need to use the internal name of the Schema and the Item Type:
* displayrule create rule FooRuleSet element.DgnCustomItemTypes_AreaAnnotator_1_0::Area__x0020__Feature::occupied=True ElementDisplay
* @bsimethod                                                              Bentley Systems
+---------------+---------------+---------------+---------------+---------------+------*/
void DisplayRuleUtils::CreateDisplayRuleSet(WCharCP unparsed)
    {
    DisplayRuleUtils displayRuleMsg;
    if (WString::IsNullOrEmpty(unparsed))
        return;

    DgnFileR dgnFile = *ISessionMgr::GetActiveDgnFile();
    DisplayRuleSetPtr displayRuleSet = DisplayRuleSet::Create(unparsed, dgnFile);
    if (displayRuleSet.IsNull())
        return;

    DisplayRulesManager::WriteDisplayRuleSetToFile(*displayRuleSet, dgnFile, true);
    displayRuleMsg.OutputMessageInMessageCenter(STRINGLISTID_Prompts, PROMPT_DisplayRuleSetCreate);
    }

/*---------------------------------------------------------------------------------**//**
* @description   Create Display Rule for an Item Type.  The rule must contain white-space-separated
* Schema Name, Item Type Name, property name, test
* e.g.
* FooRuleSet AreaAnnotator "Area Feature" Occupied =True ElementDisplay
+---------------+---------------+---------------+---------------+---------------+------*/

void DisplayRuleUtils::CreateDisplayRuleItemType(WCharCP rule)
{
    if (WString::IsNullOrEmpty(rule))
        return;
    DisplayRuleUtils 			displayRuleMsg;
	WString 					msg;
	Bstdcxx::bvector<WString>  	tokens;
    BeStringUtilities::ParseArguments(tokens, rule);
    if (tokens.size() < 6)
	{
		displayRuleMsg.OutputMessageInMessageCenter(STRINGLISTID_Prompts, PROMPT_ProvideParam);
		return;
	}
    if (WString::IsNullOrEmpty(tokens.at(0).c_str()) ||
        WString::IsNullOrEmpty(tokens.at(1).c_str()) ||
        WString::IsNullOrEmpty(tokens.at(2).c_str()) ||
        WString::IsNullOrEmpty(tokens.at(3).c_str()) ||
        WString::IsNullOrEmpty(tokens.at(4).c_str()) ||
        WString::IsNullOrEmpty(tokens.at(5).c_str()))
	{
		displayRuleMsg.OutputMessageInMessageCenter(STRINGLISTID_Prompts, PROMPT_ProvideValidParam);
		return;
	}
    WCharCP ruleSetName 		= tokens[0].GetWCharCP();
    WCharCP schemaName			= tokens[1].GetWCharCP();
    WCharCP itemTypeName		= tokens[2].GetWCharCP();
    WCharCP propName			= tokens[3].GetWCharCP();
    WCharCP ruleCondition 		= tokens[4].GetWCharCP();
    WString actionToPerform 	= tokens[5].c_str();

	msg.Sprintf (L"Rule Set '%s' schema '%s' item type '%s' property '%s' test '%s' action '%s'",
		ruleSetName, schemaName, itemTypeName, propName, ruleCondition, actionToPerform);
	displayRuleMsg.OutputMessageInMessageCenter (msg);
    DgnFileR dgnFile = *ISessionMgr::GetActiveDgnFile();
    DisplayRuleSetCP displayRuleSetCP = DisplayRulesManager::GetDisplayRuleSetByName(ruleSetName, dgnFile);
    if (nullptr == displayRuleSetCP)
    {
		msg.Sprintf (L"Rule Set '%s' is not available", ruleSetName);
		displayRuleMsg.OutputMessageInMessageCenter(msg);
		return;
	}
	//	Clone the Rule Set before modifying
    DisplayRuleSetPtr displayRuleSet = displayRuleSetCP->Clone(dgnFile);
    if (displayRuleSet.IsNull())
    {
		msg.Sprintf (L"Unable to clone Rule Set '%s' ", ruleSetName);
		displayRuleMsg.OutputMessageInMessageCenter(msg);
		return;
	}
    if (actionToPerform == ELEMENTDISPLAY) /*ElementDisplay Action*/
	{
		WString composedRule;
		composedRule.Sprintf (L"element.DgnCustomItemTypes_%s::%s::%s%s",
			GetInternalName (schemaName).c_str (),
			GetInternalName (itemTypeName).c_str (),
			GetInternalName (propName).c_str (),
			ruleCondition);
		msg.Sprintf (L"Create Rule='%s'", composedRule.c_str ());
		displayRuleMsg.OutputMessageInMessageCenter(msg);
		DisplayRulePtr      displayRuleElement = DisplayRule::Create(composedRule.c_str (), true, dgnFile);
		if (displayRuleElement.IsNull())
			return;
		displayRuleElement->GetActions().push_back(ElementDisplayAction::Create(true));
		displayRuleSet->GetDisplayRules().push_back(displayRuleElement);
		if (displayRuleSet->GetDisplayRules().size() > 0)
			DisplayRulesManager::WriteDisplayRuleSetToFile(*displayRuleSet, dgnFile, false);
		displayRuleMsg.OutputMessageInMessageCenter(STRINGLISTID_Prompts, PROMPT_ElementDisplayRule);
	}

}

/*---------------------------------------------------------------------------------**//**
* @description   Create Display Rule and add display rule to the specified rule set
* @bsimethod                                                              Bentley Systems
+---------------+---------------+---------------+---------------+---------------+------*/
void DisplayRuleUtils::CreateDisplayRule(WCharCP unparsed)
    {
    DisplayRuleUtils displayRuleMsg;
    Bstdcxx::bvector<WString>  tokens;

    if (WString::IsNullOrEmpty(unparsed))
        return;

    BeStringUtilities::ParseArguments(tokens, unparsed);
    if (tokens.size() < 3)
        {
        displayRuleMsg.OutputMessageInMessageCenter(STRINGLISTID_Prompts, PROMPT_ProvideParam);
        return;
        }

    if (WString::IsNullOrEmpty(tokens.at(0).c_str()) ||
        WString::IsNullOrEmpty(tokens.at(1).c_str()) ||
        WString::IsNullOrEmpty(tokens.at(2).c_str()))
        {
        displayRuleMsg.OutputMessageInMessageCenter(STRINGLISTID_Prompts, PROMPT_ProvideValidParam);
        return;
        }

    WCharCP ruleSetName = tokens[0].GetWCharCP();
    WCharCP ruleCondition = tokens[1].GetWCharCP();
    WString actionToPerform = tokens[2].c_str();

    DgnFileR dgnFile = *ISessionMgr::GetActiveDgnFile();
    DisplayRuleSetCP displayRuleSetCP = DisplayRulesManager::GetDisplayRuleSetByName(ruleSetName, dgnFile);
    if (nullptr == displayRuleSetCP)
        return;

    DisplayRuleSetPtr displayRuleSet = displayRuleSetCP->Clone(dgnFile);
    if (displayRuleSet.IsNull())
        return;
    /*Creates Display Rule for specified action*/
    if (actionToPerform == ELEMENTDISPLAY) /*ElementDisplay Action*/
        {
        DisplayRulePtr      displayRuleElement = DisplayRule::Create(ruleCondition, true, dgnFile);
        if (displayRuleElement.IsNull())
            return;
        displayRuleElement->GetActions().push_back(ElementDisplayAction::Create(true));
        displayRuleSet->GetDisplayRules().push_back(displayRuleElement);
        if (displayRuleSet->GetDisplayRules().size() > 0)
            DisplayRulesManager::WriteDisplayRuleSetToFile(*displayRuleSet, dgnFile, false);
        displayRuleMsg.OutputMessageInMessageCenter(STRINGLISTID_Prompts, PROMPT_ElementDisplayRule);
        }
    else if (actionToPerform == COLOROVERRIDE) /*ColorOverride Action*/
        {
        UInt32 colorOverride = 5;
        DisplayRulePtr displayRule = DisplayRule::Create(ruleCondition, true, dgnFile);
        if (displayRule.IsNull())
            return;
        displayRule->GetActions().push_back(ColorOverrideAction::Create(colorOverride, dgnFile));
        displayRuleSet->GetDisplayRules().push_back(displayRule);
        DisplayRulesManager::WriteDisplayRuleSetToFile(*displayRuleSet, dgnFile, true);
        displayRuleMsg.OutputMessageInMessageCenter(STRINGLISTID_Prompts, PROMPT_ColorOverrideDisplayRule);
        }
    else if (actionToPerform == FILLCOLOROVERRIDE) /*FillColorOverride*/
        {
        UInt32 fillColorOverride = 6;
        DisplayRulePtr      displayRule = DisplayRule::Create(ruleCondition, true, dgnFile);
        if (displayRule.IsNull())
            return;
        displayRule->GetActions().push_back(FillColorOverrideAction::Create(fillColorOverride, dgnFile));
        displayRuleSet->GetDisplayRules().push_back(displayRule);
        if (displayRuleSet->GetDisplayRules().size() > 0)
            DisplayRulesManager::WriteDisplayRuleSetToFile(*displayRuleSet, dgnFile, false);
        displayRuleMsg.OutputMessageInMessageCenter(STRINGLISTID_Prompts, PROMPT_FillColorOverrideDisplayRule);
        }
    else if (actionToPerform == STYLE) /*StyleOverride*/
        {
        UInt32 lStyleOverride = 7;
        DisplayRulePtr      displayRule = DisplayRule::Create(ruleCondition, true, dgnFile);
        if (displayRule.IsNull())
            return;
        displayRule->GetActions().push_back(StyleOverrideAction::Create(lStyleOverride, dgnFile));
        displayRuleSet->GetDisplayRules().push_back(displayRule);
        if (displayRuleSet->GetDisplayRules().size() > 0)
            DisplayRulesManager::WriteDisplayRuleSetToFile(*displayRuleSet, dgnFile, false);
        displayRuleMsg.OutputMessageInMessageCenter(STRINGLISTID_Prompts, PROMPT_StyleOverrideDisplayRule);
        }
    else if (actionToPerform == TRANSPERRENCY) /*TransparencyOverride*/
        {
        double transparencyOverride = 0.8;
        DisplayRulePtr      displayRule = DisplayRule::Create(ruleCondition, true, dgnFile);
        if (displayRule.IsNull())
            return;
        displayRule->GetActions().push_back(TransparencyOverrideAction::Create(transparencyOverride));
        displayRuleSet->GetDisplayRules().push_back(displayRule);
        if (displayRuleSet->GetDisplayRules().size() > 0)
            DisplayRulesManager::WriteDisplayRuleSetToFile(*displayRuleSet, dgnFile, false);
        displayRuleMsg.OutputMessageInMessageCenter(STRINGLISTID_Prompts, PROMPT_TransparencyOverrideDisplayRule);
        }
    else if (actionToPerform == ELEMENTPRIORITY) /*ElementPriority*/
        {
        int elementPriority = 6;
        DisplayRulePtr      displayRule = DisplayRule::Create(ruleCondition, true, dgnFile);
        if (displayRule.IsNull())
            return;
        displayRule->GetActions().push_back(ElementPriorityAction::Create(elementPriority));
        displayRuleSet->GetDisplayRules().push_back(displayRule);
        if (displayRuleSet->GetDisplayRules().size() > 0)
            DisplayRulesManager::WriteDisplayRuleSetToFile(*displayRuleSet, dgnFile, false);
        displayRuleMsg.OutputMessageInMessageCenter(STRINGLISTID_Prompts, PROMPT_PriorityOverrideDisplayRule);
        }
    }

/*---------------------------------------------------------------------------------**//**
* @description   Reads Display Rule's action value and shows it in message centre
* @bsimethod                                                              Bentley Systems
+---------------+---------------+---------------+---------------+---------------+------*/
void DisplayRuleUtils::ReadDisplayRuleFromRuleSet(WCharCP unparsed)
    {
    DisplayRuleUtils displayRuleMsg;
    Bstdcxx::bvector<WString>  tokens;

    if (WString::IsNullOrEmpty(unparsed))
        return;

    BeStringUtilities::ParseArguments(tokens, unparsed);
    if (tokens.size() < 3)
        {
        displayRuleMsg.OutputMessageInMessageCenter(STRINGLISTID_Prompts, PROMPT_ProvideValidParam);
        return;
        }

    if (WString::IsNullOrEmpty(tokens.at(0).c_str()) ||
        WString::IsNullOrEmpty(tokens.at(1).c_str()) ||
        WString::IsNullOrEmpty(tokens.at(2).c_str()))
        {
        displayRuleMsg.OutputMessageInMessageCenter(STRINGLISTID_Prompts, PROMPT_ProvideParam);
        return;
        }
    WCharCP ruleSetName = tokens[0].GetWCharCP();
    WString ruleCondition = tokens[1].c_str();
    WString actionToPerform = tokens[2].c_str();

    DgnFileR dgnFile = *ISessionMgr::GetActiveDgnFile();
    DisplayRuleSetCP displayRuleSetCP = DisplayRulesManager::GetDisplayRuleSetByName(ruleSetName, dgnFile);
    if (nullptr == displayRuleSetCP)
        return;

    DisplayRuleSetPtr displayRuleSet = displayRuleSetCP->Clone(dgnFile);
    if (displayRuleSet.IsNull())
        return;
    DisplayRulePtrVector displayRuleVec = displayRuleSet->GetDisplayRules();
    /*Iterate over Display Rule Vector*/
    for (DisplayRulePtr displayRule : displayRuleVec)
        {
        if (displayRule.IsNull())
            return;
        WStringCR  displayRuleCondition = displayRule->GetCondition();
        DisplayRuleActionPtrVector actions = displayRule->GetActions();
        /*Iterate over Display Rule actions*/
        for (IDisplayRuleActionPtr action : actions)
            {
            if (action->GetActionId() == DisplayRuleActionId::ElementDisplay && displayRuleCondition == ruleCondition && actionToPerform == ELEMENTDISPLAY)
               {
                ElementDisplayActionP act = dynamic_cast <ElementDisplayActionP> (action.get());
                bool value = act->IsElementDisplayOff();
                WString message;
                mdlResource_loadWString(message, NULL, STRINGLISTID_Prompts, PROMPT_ElementDisplayValue);
                message.Sprintf(message.c_str(), value ? L"true" : L"false");
                mdlOutput_messageCenter(OutputMessagePriority::Info, message.c_str(), NULL, OutputMessageAlert::None);
               }
            else if (action->GetActionId() == DisplayRuleActionId::ColorOverride && displayRuleCondition == ruleCondition && actionToPerform == COLOROVERRIDE)
               {
                ColorOverrideActionP act = dynamic_cast <ColorOverrideActionP> (action.get());
                UInt32 value = act->GetElementColor();
                WString message;
                mdlResource_loadWString(message, NULL, STRINGLISTID_Prompts, PROMPT_ColorOverrideValue);
                message.Sprintf(message.c_str(), value);
                mdlOutput_messageCenter(OutputMessagePriority::Info, message.c_str(), NULL, OutputMessageAlert::None);
               }
            else if (action->GetActionId() == DisplayRuleActionId::ElementPriority && displayRuleCondition == ruleCondition && actionToPerform == ELEMENTPRIORITY)
                {
                ElementPriorityActionP act = dynamic_cast<ElementPriorityActionP> (action.get());
                int value = act->GetElementPriority();
                WString message;
                mdlResource_loadWString(message, NULL, STRINGLISTID_Prompts, PROMPT_PriorityOverrideValue);
                message.Sprintf(message.c_str(), value);
                mdlOutput_messageCenter(OutputMessagePriority::Info, message.c_str(), NULL, OutputMessageAlert::None);
                }
            }
        }
    }

/*---------------------------------------------------------------------------------**//**
* @description   Modify Display Rule And Actions from the Display Rule Set and writes
                 modified display rule and rule set in dgn file
* @bsimethod                                                              Bentley Systems
+---------------+---------------+---------------+---------------+---------------+------*/
void DisplayRuleUtils::ModifyDisplayRuleAndActions(WCharCP unparsed)
    {
    DisplayRuleUtils displayRuleMsg;
    Bstdcxx::bvector<WString>  tokens;

    if (WString::IsNullOrEmpty(unparsed))
        return;

    BeStringUtilities::ParseArguments(tokens, unparsed);
    if (tokens.size() < 4)
        {
        displayRuleMsg.OutputMessageInMessageCenter(STRINGLISTID_Prompts, PROMPT_ProvideParam);
        return;
        }

    if (WString::IsNullOrEmpty(tokens.at(0).c_str()) ||
        WString::IsNullOrEmpty(tokens.at(1).c_str()) ||
        WString::IsNullOrEmpty(tokens.at(2).c_str()) ||
        WString::IsNullOrEmpty(tokens.at(3).c_str()))
        {
        displayRuleMsg.OutputMessageInMessageCenter(STRINGLISTID_Prompts, PROMPT_ProvideValidParam);
        return;
        }

    WCharCP ruleSetName = tokens[0].GetWCharCP();
    WString ruleCondition = tokens[1].c_str();
    WString actionToPerform = tokens[2].c_str();
    int modifiedParameter = BeStringUtilities::Wtoi(tokens[3].GetWCharCP());

    DgnFileR dgnFile = *ISessionMgr::GetActiveDgnFile();
    DisplayRuleSetCP displayRuleSetCP = DisplayRulesManager::GetDisplayRuleSetByName(ruleSetName, dgnFile);
    if (nullptr == displayRuleSetCP)
        return;

    DisplayRuleSetPtr displayRuleSet = displayRuleSetCP->Clone(dgnFile);
    if (displayRuleSet.IsNull())
        return;
    DisplayRulePtrVector displayRuleVec = displayRuleSet->GetDisplayRules();
    //Iterate over Display Rule Vector and fetch the
    //actions of it which we need to modify
    for (DisplayRulePtr displayRule : displayRuleVec)
        {
        if (displayRule.IsNull())
            return;
         WStringCR  displayRuleCondition = displayRule->GetCondition();
         DisplayRuleActionPtrVector actions = displayRule->GetActions();
         /*Iterate over Display Rule actions*/
         for (IDisplayRuleActionPtr action : actions)
             {
             if (action->GetActionId() == DisplayRuleActionId::ElementDisplay && displayRuleCondition == ruleCondition && actionToPerform == ELEMENTDISPLAY)
                {
                ElementDisplayActionP act = dynamic_cast <ElementDisplayActionP> (action.get());
                act->SetIsElementDisplayOff(true);
                DisplayRulesManager::WriteDisplayRuleSetToFile(*displayRuleSet, dgnFile, true);
                displayRuleMsg.OutputMessageInMessageCenter(STRINGLISTID_Prompts,PROMPT_DisplayRuleModified);
                }
             else if (action->GetActionId() == DisplayRuleActionId::ColorOverride && displayRuleCondition == ruleCondition && actionToPerform == COLOROVERRIDE)
                {
                ColorOverrideActionP act = dynamic_cast <ColorOverrideActionP> (action.get());
                act->SetElementColor(modifiedParameter);
                DisplayRulesManager::WriteDisplayRuleSetToFile(*displayRuleSet, dgnFile, true);
                displayRuleMsg.OutputMessageInMessageCenter(STRINGLISTID_Prompts, PROMPT_DisplayRuleModified);
                }
             else if (action->GetActionId() == DisplayRuleActionId::FillColorOverride && displayRuleCondition == ruleCondition && actionToPerform == FILLCOLOROVERRIDE)
                {
                FillColorOverrideActionP act = dynamic_cast<FillColorOverrideActionP> (action.get());
                act->SetElementFillColor(modifiedParameter);
                DisplayRulesManager::WriteDisplayRuleSetToFile(*displayRuleSet, dgnFile, true);
                displayRuleMsg.OutputMessageInMessageCenter(STRINGLISTID_Prompts, PROMPT_DisplayRuleModified);
                }
             else if (action->GetActionId() == DisplayRuleActionId::StyleOverride && displayRuleCondition == ruleCondition && actionToPerform == STYLE)
                {
                StyleOverrideActionP act = dynamic_cast<StyleOverrideActionP> (action.get());
                act->SetLineStyle(modifiedParameter);
                DisplayRulesManager::WriteDisplayRuleSetToFile(*displayRuleSet, dgnFile, true);
                displayRuleMsg.OutputMessageInMessageCenter(STRINGLISTID_Prompts, PROMPT_DisplayRuleModified);
                }
             else if (action->GetActionId() == DisplayRuleActionId::TransparencyOverride && displayRuleCondition == ruleCondition && actionToPerform == TRANSPERRENCY)
                {
                TransparencyOverrideActionP act = dynamic_cast<TransparencyOverrideActionP> (action.get());
                act->SetTransparency(modifiedParameter);
                DisplayRulesManager::WriteDisplayRuleSetToFile(*displayRuleSet, dgnFile, true);
                displayRuleMsg.OutputMessageInMessageCenter(STRINGLISTID_Prompts, PROMPT_DisplayRuleModified);
                }
             else if (action->GetActionId() == DisplayRuleActionId::ElementPriority && displayRuleCondition == ruleCondition && actionToPerform == ELEMENTPRIORITY)
                {
                ElementPriorityActionP act = dynamic_cast<ElementPriorityActionP> (action.get());
                act->SetElementPriority(modifiedParameter);
                DisplayRulesManager::WriteDisplayRuleSetToFile(*displayRuleSet, dgnFile, true);
                displayRuleMsg.OutputMessageInMessageCenter(STRINGLISTID_Prompts, PROMPT_DisplayRuleModified);
                }
             }
        }
    }

/*---------------------------------------------------------------------------------**//**
* @description   Delete Display Rule Set from the dgn file
* @bsimethod                                                              Bentley Systems
+---------------+---------------+---------------+---------------+---------------+------*/
void DisplayRuleUtils::DeleteDisplayRuleSet(WCharCP unparsed)
    {
    DisplayRuleUtils displayRuleMsg;
    if (WString::IsNullOrEmpty(unparsed))
        return;

    DgnFileR dgnFile = *ISessionMgr::GetActiveDgnFile();
    DisplayRulesManager::RemoveDisplayRuleSetFromFile(unparsed, dgnFile);
    displayRuleMsg.OutputMessageInMessageCenter(STRINGLISTID_Prompts, PROMPT_DisplayRuleSetDelete);
    }

/*---------------------------------------------------------------------------------**//**
* @description   Delete display rule from display rule set and writes updated rule set to dgn file
* @bsimethod                                                              Bentley Systems
+---------------+---------------+---------------+---------------+---------------+------*/
void DisplayRuleUtils::DeleteDisplayRule(WCharCP unparsed)
    {
    DisplayRuleUtils displayRuleMsg;
    Bstdcxx::bvector<WString>  tokens;

    if (WString::IsNullOrEmpty(unparsed))
        return;

    BeStringUtilities::ParseArguments(tokens, unparsed);
    if (tokens.size() < 2)
        {
        displayRuleMsg.OutputMessageInMessageCenter(STRINGLISTID_Prompts, PROMPT_ProvideParam);
        return;
        }

    if (WString::IsNullOrEmpty(tokens.at(0).c_str()) ||
        WString::IsNullOrEmpty(tokens.at(1).c_str()))
        {
        displayRuleMsg.OutputMessageInMessageCenter(STRINGLISTID_Prompts, PROMPT_ProvideValidParam);
        return;
        }

    WCharCP ruleSetName = tokens[0].GetWCharCP();
    WString ruleCondition = tokens[1].c_str();

    DgnFileR dgnFile = *ISessionMgr::GetActiveDgnFile();
    DisplayRuleSetCP displayRuleSetCP = DisplayRulesManager::GetDisplayRuleSetByName(ruleSetName, dgnFile);
    if (nullptr == displayRuleSetCP)
        return;

    DisplayRuleSetPtr displayRuleSet = displayRuleSetCP->Clone(dgnFile);
    if (displayRuleSet.IsNull())
        return;

    //Iterate over Display Rules from Display Rule Set
    for (DisplayRulePtrVector::iterator iter = displayRuleSet->GetDisplayRules().begin(); iter != displayRuleSet->GetDisplayRules().end(); ++iter)
        {
        WStringCR  displayRuleCondition = (*iter)->GetCondition();
        if (0 == displayRuleCondition.compare(ruleCondition))
            {
            //Erase Display Rule from Rule Set
            displayRuleSet->GetDisplayRules().erase(iter);
            DisplayRulesManager::WriteDisplayRuleSetToFile(*displayRuleSet, dgnFile, true);
            break;
            }
        }
    displayRuleMsg.OutputMessageInMessageCenter(STRINGLISTID_Prompts, PROMPT_DisplayRuleDeleted);
    }






















